@@ -42,7 +42,7 @@ In ${CONF}/slackpkg.conf.new, there is a sample of the new configuration.\n\
 
 	# Checking if another instance of slackpkg is running
 	#
-	if [ "`ls /var/lock/slackpkg.* 2>/dev/null`" ] && \
+	if [ "$(ls /var/lock/slackpkg.* 2>/dev/null)" ] && \
 		[ "$CMD" != "search" ]; then
 		echo -e "\
 \nAnother instance of slackpkg is running. If this is not correct, you can\n\
@@ -86,7 +86,7 @@ Please edit that file and uncomment ONE mirror.  Slackpkg\n\
 only works with ONE mirror selected.\n"
 		cleanup
 	else
-		COUNT=`echo $SOURCE | wc -w | tr -d " "`
+		COUNT=$(echo $SOURCE | wc -w | tr -d " ")
 		if [ "$COUNT" != "1" ]; then
 			echo -e "\n\
 Slackpkg only works with ONE mirror selected.  Please edit your\n\
@@ -107,21 +107,9 @@ official mirrors can be kept up-to-date.\n"
 		cleanup
 	fi
 
-	# Command line syntax checking 
-	#
-	if [ "$#" = "0" ]; then
-		usage
-	fi
-
-	if [ "$#" = "1" ] && [ "$CMD" != "update" ] && \
-		[ "$CMD" != "upgrade-all" ] && [ "$CMD" != "install-new" ] && \
-		[ "$CMD" != "blacklist" ] && [ "$CMD" != "clean-system" ]; then
-		usage
-	fi
-	
 	# Checking if the user has the permissions to install/upgrade/update
 	#                                                                    
-	if [ "`id -u`" != "0" ] && [ "$CMD" != "search" ] && [ "$CMD" != "info" ]; then
+	if [ "$(id -u)" != "0" ] && [ "$CMD" != "search" ] && [ "$CMD" != "info" ]; then
 		echo -e "\n\
 Only root can install, upgrade, or remove packages.\n\
 Please log in as root or contact your system administrator.\n"
@@ -130,16 +118,25 @@ Please log in as root or contact your system administrator.\n"
 
 	# Check if the mirror are local (cdrom or file)
 	#
-	MEDIA=`echo ${SOURCE} | cut -f1 -d:`
-	if [ "$MEDIA" = "cdrom" ] || [ "$MEDIA" = "file" ]; then
-		SOURCE=/`echo ${SOURCE} | cut -f3- -d/`
+	MEDIA=$(echo ${SOURCE} | cut -f1 -d:)
+	if [ "$MEDIA" = "cdrom" ] || [ "$MEDIA" = "file" ] || \
+	   [ "$MEDIA" = "local" ]; then
+		SOURCE=/$(echo ${SOURCE} | cut -f3- -d/)
 		LOCAL=1
 	fi
 
+	# Check if the "which" command is there
+	if ! which which 1>/dev/null 2>/dev/null ; then
+		echo -e "\n\
+No 'which' command found, please install it if you want to\n\
+use slackpkg.\n"
+		cleanup
+	fi
+
 	# Check if we have md5sum in the PATH. Without md5sum, disables
 	# md5sum checks
 	#
-	if ! [ `which md5sum 2>/dev/null` ]; then
+	if ! [ $(which md5sum 2>/dev/null) ]; then
 		CHECKPKG=off
 	elif ! [ -f ${WORKDIR}/CHECKSUMS.md5 ] && \
 		[ "$CMD" != "update" ] && \
@@ -175,8 +172,8 @@ with slackpkg.\n"
 
 	# Check if the Slackware GPG key are found in the system
 	#                                                       
-	GPGFIRSTTIME="`gpg --list-keys \"$SLACKKEY\" 2>/dev/null \
-			| grep -c \"$SLACKKEY\"`"
+	GPGFIRSTTIME="$(gpg --list-keys \"$SLACKKEY\" 2>/dev/null \
+			| grep -c "$SLACKKEY")"
 	if [ "$GPGFIRSTTIME" = "0" ] && [ "$CMD" != "search" ] && [ "$CMD" != "info" ] && \
 			[ "$CMD" != "update" ] && [ "$CHECKGPG" = "on" ]; then
 		echo -e "\n\
@@ -189,6 +186,16 @@ to "off" - you can see an example in the original slackpkg.conf.new file distrib
 with slackpkg.\n"
 		cleanup
 	fi
+
+	if [ "$BATCH" = "on" ] || [ "$BATCH" = "ON" ]; then
+		DIALOG=off
+		MORE=cat
+		if [ "$DEFAULT_ANSWER" = "" ]; then
+			DEFAULT_ANSWER=n
+		fi
+	else
+		MORE=more
+	fi
 	echo 
 }
 
@@ -218,12 +225,13 @@ slackpkg - version $VERSION\n\
 \t\t\t\t\talready installed
 \tslackpkg remove package\t\tremove installed packages
 \tslackpkg clean-system\t\tremove all packages which are not 
-\t\t\t\t\tpresent in the official Slackware package set.
-\t\t\t\t\tGood to keep the house in order
+\t\t\t\t\tpresent in the official Slackware 
+\t\t\t\t\tpackage set. Good to keep the house
+\t\t\t\t\tin order
 \tslackpkg upgrade-all\t\tsync all packages installed in your 
 \t\t\t\t\tmachine with the selected mirror. This
-\t\t\t\t\tis the "correct" way to upgrade all of your 
-\t\t\t\t\tmachine.
+\t\t\t\t\tis the "correct" way to upgrade all of 
+\t\t\t\t\tyour machine.
 \tslackpkg install-new\t\tinstall packages which are added to
 \t\t\t\t\tthe official Slackware package set.
 \t\t\t\t\tRun this if you are upgrading to another
@@ -236,6 +244,8 @@ slackpkg - version $VERSION\n\
 \t\t\t\t\t(works with only ONE package)
 \tslackpkg search file\t\tSearch for a specific file in the
 \t\t\t\t\tentire package collection
+\tslackpkg new-config\t\tSearch for new configuration files and
+\t\t\t\t\task to user what to do with them.
 \nYou can see more information about slackpkg usage and some examples
 in slackpkg's manpage. You can use partial package names (such as x11
 instead x11-devel, x11-docs, etc), or even Slackware series
@@ -250,8 +260,8 @@ function checkpkg() {
 	local MD5ORIGINAL
 	local MD5DOWNLOAD
 
-	MD5ORIGINAL=`grep "/${NAMEPKG}$" ${WORKDIR}/CHECKSUMS.md5| cut -f1 -d \ `
-	MD5DOWNLOAD=`md5sum ${TEMP}/${1} | cut -f1 -d \ `
+	MD5ORIGINAL=$(grep "/${NAMEPKG}$" ${WORKDIR}/CHECKSUMS.md5| cut -f1 -d \ )
+	MD5DOWNLOAD=$(md5sum ${TEMP}/${1} | cut -f1 -d \ )
 	if [ "$MD5ORIGINAL" = "$MD5DOWNLOAD" ]; then
 		echo 1 
 	else
@@ -292,12 +302,12 @@ function givepriority {
 	
         for DIR in $FIRST $SECOND $THIRD $FOURTH $FIFTH ; do
 		[ "$PKGDATA" ] && break
-                PKGDATA=( `grep "^${DIR} ${ARGUMENT} " ${WORKDIR}/pkglist` )
+                PKGDATA=( $(grep "^${DIR} ${ARGUMENT} " ${WORKDIR}/pkglist) )
                 if [ "$PKGDATA" ]; then
                         checkblacklist
                         if [ "$?" = "1" ]; then
 				NAME=${PKGDATA[1]}
-                                FULLNAME=`echo "${PKGDATA[5]}"`
+                                FULLNAME=$(echo "${PKGDATA[5]}")
 			else
 				unset PKGDATA
 				unset FULLNAME
@@ -336,31 +346,31 @@ function makelist() {
 
 	case "$CMD" in
 		download)
-			for ARGUMENT in `echo $INPUTLIST`; do
-				for i in `grep -w -- "${ARGUMENT}" ${WORKDIR}/pkglist | cut -f2 -d\  | sort -u`; do
-					LIST="$LIST `grep " ${i} " ${WORKDIR}/pkglist | cut -f6 -d \ `"
+			for ARGUMENT in $(echo $INPUTLIST); do
+				for i in $(grep -w -- "${ARGUMENT}" ${WORKDIR}/pkglist | cut -f2 -d\  | sort -u); do
+					LIST="$LIST $(grep " ${i} " ${WORKDIR}/pkglist | cut -f6 -d \ )"
 				done
-				LIST="`echo -e $LIST | sort -u`"
+				LIST="$(echo -e $LIST | sort -u)"
 			done
 		;;
 		blacklist)
-			for ARGUMENT in `echo $INPUTLIST`; do
-				for i in `cat ${WORKDIR}/pkglist ${TMPDIR}/tmplist | \
-						grep -w -- "${ARGUMENT}" | cut -f2 -d\  | sort -u`; do
+			for ARGUMENT in $(echo $INPUTLIST); do
+				for i in $(cat ${WORKDIR}/pkglist ${TMPDIR}/tmplist | \
+						grep -w -- "${ARGUMENT}" | cut -f2 -d\  | sort -u); do
 					grep -qx "${i}" ${CONF}/blacklist || LIST="$LIST $i"
 				done
 			done
 		;;
 		install|upgrade|reinstall)
-			for ARGUMENT in `echo $INPUTLIST`; do
-				for i in `grep -w -- "${ARGUMENT}" ${WORKDIR}/pkglist | cut -f2 -d\  | sort -u`; do
+			for ARGUMENT in $(echo $INPUTLIST); do
+				for i in $(grep -w -- "${ARGUMENT}" ${WORKDIR}/pkglist | cut -f2 -d\  | sort -u); do
 					givepriority $i
 					[ ! "$FULLNAME" ] && continue
 
 					case $CMD in
 						'upgrade')
-							VRFY=`cut -f6 -d\  ${TMPDIR}/tmplist | \
-							      grep -x "${NAME}-[^-]\+-\(noarch\|${ARCH}\)-[^-]\+"`
+							VRFY=$(cut -f6 -d\  ${TMPDIR}/tmplist | \
+							      grep -x "${NAME}-[^-]\+-\(noarch\|fw\|${ARCH}\)-[^-]\+")
 							[ "${FULLNAME}" != "${VRFY}" ]  && \
 										[ "${VRFY}" ] && \
 								LIST="$LIST ${FULLNAME}"
@@ -378,10 +388,10 @@ function makelist() {
 			done
 		;;
 		remove)
-			for ARGUMENT in `echo $INPUTLIST`; do
-				for i in `cat ${WORKDIR}/pkglist ${TMPDIR}/tmplist | \
-					  	grep -w -- "${ARGUMENT}" | cut -f6 -d\  | sort -u`; do
-					PKGDATA=( `grep -w -- "$i" ${TMPDIR}/tmplist` )
+			for ARGUMENT in $(echo $INPUTLIST); do
+				for i in $(cat ${WORKDIR}/pkglist ${TMPDIR}/tmplist | \
+					  	grep -w -- "${ARGUMENT}" | cut -f6 -d\  | sort -u); do
+					PKGDATA=( $(grep -w -- "$i" ${TMPDIR}/tmplist) )
 					[ ! "$PKGDATA" ] && continue
 					checkblacklist
 					LIST="$LIST ${PKGDATA[5]}" 
@@ -390,29 +400,31 @@ function makelist() {
 			done
 		;;
 		clean-system)
-	                for i in `cut -f6 -d\  ${TMPDIR}/tmplist`; do
-				NAME=`cutpkg $i`
-				if [ `cut -f2 -d\  ${WORKDIR}/pkglist |grep -cx "${NAME}"` = "0" ] &&
-					[ `grep -cx "${NAME}" $CONF/blacklist` = 0 ]; then
+	                for i in $(cut -f6 -d\  ${TMPDIR}/tmplist); do
+				NAME=$(cutpkg $i)
+				if [ $(cut -f2 -d\  ${WORKDIR}/pkglist |grep -cx "${NAME}") = "0" ] &&
+					[ $(grep -cx "${NAME}" $CONF/blacklist) = 0 ]; then
 					LIST="$LIST $i"
                         	fi
                 	done
 		;;
 		upgrade-all)
-			for i in `cut -f2 -d\  ${TMPDIR}/tmplist`; do
+			for i in $(cut -f2 -d\  ${TMPDIR}/tmplist); do
 
 				givepriority ${i}
 				[ ! "$FULLNAME" ] && continue
 
-				VRFY=`cut -f6 -d\  ${TMPDIR}/tmplist | grep -x "${NAME}-[^-]\+-\(noarch\|${ARCH}\)-[^-]\+"`
+				VRFY=$(cut -f6 -d\  ${TMPDIR}/tmplist | grep -x "${NAME}-[^-]\+-\(noarch\|fw\|${ARCH}\)-[^-]\+")
 				[ "${FULLNAME}" != "${VRFY}" ]  && \
 							[ "${VRFY}" ] && \
 					LIST="$LIST ${FULLNAME}"
 			done
 		;;
 		install-new)
-			for i in `awk -f /usr/libexec/slackpkg/install-new.awk ${WORKDIR}/ChangeLog.txt |\
-				  sort -u ` dialog aaa_terminfo fontconfig ; do
+			for i in $(awk -f /usr/libexec/slackpkg/install-new.awk ${WORKDIR}/ChangeLog.txt |\
+				  sort -u ) dialog aaa_terminfo fontconfig \
+				ntfs-3g ghostscript wqy-zenhei-font-ttf \
+				xbacklight xf86-video-geode ; do
 	
 				givepriority $i
 				[ ! "$FULLNAME" ] && continue
@@ -422,20 +434,29 @@ function makelist() {
 			done
 		;;
 	esac
-	LIST=`echo -e $LIST | tr \  "\n"`
+	LIST=$(echo -e $LIST | tr \  "\n" | uniq )
 	echo -e "DONE\n"
 }
 
 # Function to count total of packages
 #
 function countpkg() {
-	local COUNTPKG=`echo -e "$1" | wc -w`
+	local COUNTPKG=$(echo -e "$1" | wc -w)
 
 	if [ "$COUNTPKG" != "0" ]; then
 		echo -e "Total package(s): $COUNTPKG\n"
 	fi
 }
 
+function answer() {
+	if [ "$BATCH" = "on" ]; then
+		ANSWER="$DEFAULT_ANSWER"
+		echo $DEFAULT_ANSWER
+	else
+		read ANSWER
+	fi
+}
+
 # Show the lists and asks if the user want to proceed with that action
 # Return accepted list in $SHOWLIST
 #
@@ -443,10 +464,11 @@ function showlist() {
 	local ANSWER
 	local i
 
-	for i in $1; do echo $i; done | more 
+	for i in $1; do echo $i; done | $MORE 
 	echo
 	countpkg "$1"
-	echo -e "Do you wish to $2 selected packages (Y/n)? \c"; read ANSWER
+	echo -e "Do you wish to $2 selected packages (Y/n)? \c"
+	answer
 	if [ "$ANSWER" = "N" -o "$ANSWER" = "n" ]; then
 		cleanup
 	else
@@ -474,7 +496,7 @@ function getpkg() {
 	local FULLPATH
 	local NAMEPKG
 
-	PKGNAME=( `grep -w -m 1 -- "$1" ${WORKDIR}/pkglist` )
+	PKGNAME=( $(grep -w -m 1 -- "$1" ${WORKDIR}/pkglist) )
 	NAMEPKG=${PKGNAME[5]}
 	FULLPATH=${PKGNAME[6]}
 
@@ -511,7 +533,7 @@ function getpkg() {
 	# packages md5sum to detect if they are corrupt or not
 	#
 	if [ "$CHECKPKG" = "on" ] && [ "$ISOK" = "1" ]; then
-		ISOK=`checkpkg $1`
+		ISOK=$(checkpkg $1)
 		if [ "$ISOK" = "0" ]; then 
 			ERROR="md5sum"
 			echo -e "${NAMEPKG}:\t$ERROR" >> $TMPDIR/error.log
@@ -522,7 +544,7 @@ function getpkg() {
 	# disable GPG checking in /etc/slackpkg/slackpkg.conf
 	#
 	if [ "$CHECKGPG" = "on" ] && [ "$ISOK" = "1" ]; then
-		ISOK=`checkgpg $1`
+		ISOK=$(checkgpg $1)
 		if [ "$ISOK" = "0" ]; then 
 			ERROR="gpg"
 			echo -e "${NAMEPKG}:\t$ERROR" >> $TMPDIR/error.log
@@ -569,11 +591,18 @@ function updatefilelists()
 	# or different from our already existent ChangeLog.txt 
 	#
 	getfile ChangeLog.txt $TMPDIR/ChangeLog.txt
+	if ! grep -q "[a-z]" $TMPDIR/ChangeLog.txt ; then
+		echo -e "\
+\nError downloading from $SOURCE.\n\
+Please, check your mirror and try again."
+		cleanup
+	fi
+
 	if diff --brief ${WORKDIR}/ChangeLog.txt $TMPDIR/ChangeLog.txt ; then
 		echo -e "\
 \n\t\tNo changes in ChangeLog.txt between your last update and now.\n\
 \t\tDo you really want to download all other files (y/N)? \c"
-		read ANSWER
+		answer
 		if [ "$ANSWER" != "Y" ] && [ "$ANSWER" != "y" ]; then
 			cleanup
 		fi
@@ -598,8 +627,9 @@ function updatefilelists()
 
 	if [ "$CHECKPKG" = "on" ]; then
 		echo -e "\t\tChecksums"
-		getfile CHECKSUMS.md5 ${WORKDIR}/CHECKSUMS.md5
+		getfile CHECKSUMS.md5 ${TMPDIR}/CHECKSUMS.md5
 	fi
+	cp $TMPDIR/CHECKSUMS.md5 $WORKDIR/CHECKSUMS.md5
 		
 	# Download all PACKAGES.TXT files
 	# 
@@ -612,8 +642,11 @@ function updatefilelists()
 	#
 	echo -e "\tFormatting lists to slackpkg style..."
 	echo -e "\t\tPackage List"
-	grep "\.tgz" $TMPDIR/FILELIST.TXT| awk -f /usr/libexec/slackpkg/pkglist.awk > ${WORKDIR}/pkglist 
-		
+	grep "\.tgz" $TMPDIR/FILELIST.TXT| \
+		awk -f /usr/libexec/slackpkg/pkglist.awk | \
+		sed -e 's/^M//g' > ${TMPDIR}/pkglist
+	cp ${TMPDIR}/pkglist ${WORKDIR}/pkglist		
+
 	# Format MANIFEST
 	#
 		
@@ -622,8 +655,9 @@ function updatefilelists()
 	MANFILES=""
 	for i in $DIRS; do
 		bunzip2 -c $TMPDIR/${i}-MANIFEST.bz2 | awk -f /usr/libexec/slackpkg/filelist.awk | \
-			gzip > ${WORKDIR}/${i}-filelist.gz
+			gzip > ${TMPDIR}/${i}-filelist.gz
 	done
+	cp ${TMPDIR}/*-filelist.gz ${WORKDIR}/
 
 	if [ -r ${WORKDIR}/filelist.gz ]; then
 		rm ${WORKDIR}/filelist.gz
@@ -646,17 +680,17 @@ function sanity_check() {
 	local DOUBLEFILES
 	local ANSWER
 
-	for i in `ls -1 /var/log/packages | \
-		egrep -- "^.*-(${ARCH}|noarch)-[^-]+-upgraded"`; do
-		REVNAME=`echo ${i} | awk -F'-upgraded' '{ print $1 }'`
+	for i in $(ls -1 /var/log/packages | \
+		egrep -- "^.*-(${ARCH}|fw|noarch)-[^-]+-upgraded"); do
+		REVNAME=$(echo ${i} | awk -F'-upgraded' '{ print $1 }')
 		mv /var/log/packages/${i} /var/log/packages/${REVNAME}
 		mv /var/log/scripts/${i} /var/log/scripts/${REVNAME}
 	done 
-	for i in `ls -1 /var/log/packages | egrep "^.*-(${ARCH}|noarch)-[^-]+$"`; do
+	for i in $(ls -1 /var/log/packages | egrep "^.*-(${ARCH}|fw|noarch)-[^-]+$"); do
 		cutpkg $i
 	done | sort > $TMPDIR/list1
 	cat $TMPDIR/list1 | uniq > $TMPDIR/list2
-	FILES="`diff $TMPDIR/list1 $TMPDIR/list2 | grep '<' | cut -f2 -d\ `"
+	FILES="$(diff $TMPDIR/list1 $TMPDIR/list2 | grep '<' | cut -f2 -d\ )"
 	if [ "$FILES" != "" ]; then
 		for i in $FILES ; do
 			grep -qx "${i}" ${CONF}/blacklist && continue
@@ -672,7 +706,7 @@ worry about this list - when you select your action, slackpkg will show a\n\
 better list:\n"
 		for i in $DOUBLEFILES ; do
 			ls -1 /var/log/packages |\
-				egrep -i -- "^${i}-[^-]+-(${ARCH}|noarch)-"
+				egrep -i -- "^${i}-[^-]+-(${ARCH}|fw|noarch)-"
 		done
 		echo -ne "\n\
 You can (B)lacklist, (R)emove, or (I)gnore these packages.\n\
@@ -686,8 +720,8 @@ Select your action (B/R/I): "
 			;;
 			R|r)
 				for i in $DOUBLEFILES ; do
-					FILE=`ls -1 /var/log/packages |\
-						egrep -i -- "^${i}-[^-]+-(${ARCH}|noarch)-"`
+					FILE=$(ls -1 /var/log/packages |\
+						egrep -i -- "^${i}-[^-]+-(${ARCH}|fw|noarch)-")
 					FILES="$FILES $FILE"
 				done
 				showlist "$FILES" remove
